package pl.webnet.JaTrac;

import java.awt.*;

class Rectangle extends Primitive
{
	Point x1, x2, x3, x4;
	Vec normal;
	Vec tangent;
	double D;
	boolean infinite = false;

	public Rectangle(Point x1, Point x2, Point x3, Point x4)
	{
		this.x1 = new Point(x1);
		this.x2 = new Point(x2);
		this.x3 = new Point(x3);
		this.x4 = new Point(x4);

		Vec edge1 = new Vec(x1.x - x2.x, x1.y - x2.y, x1.z - x2.z);
		Vec edge2 = new Vec(x3.x - x2.x, x3.y - x2.y, x3.z - x2.z);

		normal = edge1.cross(edge2);
		normal.unit();

		tangent = new Vec(x1.x - x4.x, x1.y - x4.y, x1.z - x4.z);
		tangent.unit();

		D = - (normal.x * x2.x + normal.y * x2.y + normal.z * x2.z);
	}

	public Rectangle(Point x1, Point x2, Point x3, Point x4, boolean inf)
	{
		this(x1, x2, x3, x4);
		infinite = inf;
	}
	
	public int intersect(Ray ray)
	{
		double vd = normal.dot(ray);

		// check if ray is not parallel to the plane
		if(vd == 0.)
			return 0;

		Vec rayOrigin = new Vec(ray.origin);
	//	rayOrigin.unit();

		double v0 = - (normal.dot(rayOrigin) + D);
		double t = v0 / vd;

		// check if the intersection is not behind ray's origin
		if(t < Ray.EPSILON)
			return 0;

		if(infinite == true)
			ray.intersections.add(new Intersection(t, this, Intersection.ENTERING, null));

		return 1;
	}

	public Vec getNormal(Point p)
	{
		Vec normal = new Vec(this.normal);

		return normal;		
	}

	public Vec getTangent(Point p, Vec n)
	{
		return new Vec(tangent);
	}

	public String toString()
	{
		return(getClass().getName() + '@' + Integer.toHexString(hashCode()) + ": " + x1 + ", " + x2 + ", " + x3 + ", " + x4 + ". Normal: " + normal + ". D: " + D);
	}
}
