package pl.webnet.JaTrac;

import java.awt.*;

/**
 * Sphere implements spherical primitive.
 * @author: Miloslaw Smyk (thorgal@amiga.com.pl)
 * @version: 1.0
 */


class Sphere extends Primitive implements XmlSettable
{
	Point center;
	double radius, radiusSquared;
	static Vec unit = new Vec(0., -1., 0.);

/**
 * Construct a sphere.
 * @param		center sphere's center in world coordinates
 * @param		radius sphere's radius
 */
	public Sphere(Point center, double radius)
	{
		this.center = new Point(center);
		this.radius = radius;
		radiusSquared = radius * radius;
	}

	public Sphere()
	{
	}

	public String[] getParamArray()
	{
		final String params[] = {
			"origin", "center", "",
			"radius", "", "setRadius",
			"spectrum", "", "setSpectrum",
			"surface", "", "setSurface"
		};
		
		return params;
	}

	public void setSpectrum(String sp)
	{
		System.out.println("Looking for spectrum: " + sp);
		spectrum = (Spectrum)ObjectIndex.find(sp, Spectrum.index);
	}

	public void setSurface(String sp)
	{
		System.out.println("Looking for surface: " + sp);
		surface = (Surface)ObjectIndex.find(sp, Surface.index);
	}

	public void addObject(Object o) {}

	public void setRadius(String rad)
	{
		double r = ParamHelper.extractDouble(rad);
		radius = r;
		radiusSquared = r * r;
	}

/**
 * Check if given ray intersects the sphere. Uses geometric solution from
 * <em>An Introduction to Raytracing</em> by A. S. Glassner et all.
 * @param		ray the ray to test against
 * @return	the number of intersections
 */
	public int intersect(Ray ray)
	{
		Vec oc = new Vec(center, ray.origin);

		double b = oc.dot(ray);
		double disc = b * b - oc.dot(oc) + radiusSquared;
		int nroots = 0;
		double t1, t2;

		if(disc <= 0.)
			return 0;

		disc = Math.sqrt(disc);
		t2 = b + disc;

		if(t2 <= Ray.EPSILON)
			return 0;

		t1 = b - disc;

		if(t1 > Ray.EPSILON)
		{
			ray.intersections.add(new Intersection(t1, this, Intersection.ENTERING, null));
			nroots = 2;
		}
		else
			nroots = 1;

			ray.intersections.add(new Intersection(t2, this, Intersection.EXITING, getSurface()));

			return nroots;
	}

	public Vec getTangent(Point p, Vec n)
	{
		Vec tangent;
		double len = unit.dot(n);

		if(n == null)
			n = new Vec(p, center);

		tangent = new Vec(- len * n.x, - 1 - len * n.y, - len * n.z);

		tangent.unit();

		return tangent;
	}

	public Vec getNormal(Point p)
	{
		Vec normal = new Vec(p, center);

		normal.unit();

		return normal;		
	}

}
